require 'spec_helper'

describe Patriarch::DAOServices::TripartiteRelationshipBuilderService do
  
  let(:monster)       { Monster.create }
  let(:fallen_angel)  { FallenAngel.create }
  let(:love_letter)   { LoveLetter.create }
  let(:instance)      { Patriarch::DAOServices::TripartiteRelationshipBuilderService.instance }

  before(:each) do
    # Deals with transaction
    @transac ||= Patriarch::TransactionServices::TransactionManagerService.instance.new_transaction(:praise)
    @transac.add_step(:praise,monster,fallen_angel,love_letter)
    
    # Stubs the DAO retriever service so it gives DAO arrays back
    dao_sample = { 
      :actor => Redis::SortedSet.new("monster:#{monster.id}:patriarch_fallen_angels_i_praise_via_love_letters",:marshal => true), 
      :target => Redis::SortedSet.new("fallen_angel:#{fallen_angel.id}:patriarch_monsters_prasing_me_via_love_letters",:marshal => true),
      :medium => Redis::SortedSet.new("love_letter:#{fallen_angel.id}:patriarch_monsters_prasing_fallen_angels_via_me",:marshal => true)
    }
    Patriarch::DAOServices::RetrieverService.instance.stub(:call).exactly(3).times.with(any_args()).and_return(dao_sample) 
  end

  it "shall insert processable lambdas into queues when create is called" do
    expect{
      instance.create(@transac)
    }.to change{ @transac.transaction_queue.select{ |queue_element| queue_element.is_a? Patriarch::RedisInstruction}.size }.by(3)
  
    expect{
      @transac.transaction_queue.each do |redis_ostruct_instruction| 
        os = redis_ostruct_instruction
        dao, method_sym, args = [os.dao,os.method_sym,os.args]
        dao.send(method_sym,*args)
			end
    }.to change{ $redis.keys.size}.by(3)
  end

  it "shall insert processable lambdas into queues when destroy is called" do
    expect{
      instance.destroy(@transac)
    }.to change{ @transac.transaction_queue.select{ |queue_element| queue_element.is_a? Patriarch::RedisInstruction}.size }.by(3)

    expect{
      @transac.transaction_queue.each do |redis_ostruct_instruction|
				os = redis_ostruct_instruction                        			
        dao, method_sym, args = [os.dao,os.method_sym,os.args]
	      dao.send(method_sym,*args)
			end
    }.to change{ $redis.info["total_commands_processed"].to_i}.by(4)
  end

  after(:each) do
    $redis.del "monster:#{monster.id}:patriarch_fallen_angels_i_praise_via_love_letters"
    $redis.del "fallen_angel:#{fallen_angel.id}:patriarch_monsters_prasing_me_via_love_letters"
    $redis.del "love_letter:#{fallen_angel.id}:patriarch_monsters_prasing_fallen_angels_via_me"
  end
end
