require 'spec_helper'

describe Patriarch::DAOServices::RedisMapperService do

  context "in bipartite situations" do

    before do
      @instance = Patriarch::DAOServices::RedisMapperService.instance
      @context = {:relation_type => :like, :actor_type => :monster, :target_type => :fallen_angel, 
      :actor_id => 1, :target_id => 1}
      @transaction_item = double()
      @transaction_item.stub(:relation_type).and_return(@context[:relation_type])
      @transaction_item.stub(:actor_type).and_return(@context[:actor_type])
      @transaction_item.stub(:target_type).and_return(@context[:target_type])   
      @transaction_item.stub(:medium_type).and_return(@context[:medium_type]) 
      @transaction_item.stub(:tripartite?).and_return(false)      
    end 

    it "retrieves DAO" do
      @instance.call(@transaction_item,:actor).should eq({:type => "sorted_set", :key => "patriarch_fallen_angels_i_like"})
      @instance.call(@transaction_item,:target).should eq({:type => "sorted_set", :key => "patriarch_monsters_liking_me"})
    end
  end # bipartite context

  context "in tripartite situations" do
    before do
      @instance = Patriarch::DAOServices::RedisMapperService.instance
      @context = {:relation_type => :tease, :actor_type => :fallen_angel, :target_type => :monster, :medium_type => :love_letter,
      :actor_id => 1, :target_id => 1, :medium_id => 1}
      @transaction_item = double()
      @transaction_item.stub(:relation_type).and_return(@context[:relation_type])
      @transaction_item.stub(:actor_type).and_return(@context[:actor_type])
      @transaction_item.stub(:target_type).and_return(@context[:target_type])    
      @transaction_item.stub(:medium_type).and_return(@context[:medium_type]) 
      @transaction_item.stub(:tripartite?).and_return(true)
    end 
    
    it "retrieves DAO" do
      @instance.call(@transaction_item,:actor).should eq(
        {:type => "sorted_set", :key => "patriarch_monsters_i_tease_via_love_letters", :options => {:marshal => true}}
      )
      @instance.call(@transaction_item,:target).should eq(
        {:type => "sorted_set", :key => "patriarch_fallen_angels_teasing_me_via_love_letters",:options => {:marshal => true}}
      )
      @instance.call(@transaction_item,:medium).should eq(
        {:type => "sorted_set", :key => "patriarch_fallen_angels_teasing_monsters_via_me",:options => {:marshal => true}}
      )
    end
  end #tripartite context
end # describe


    
    
