require 'spec_helper'

describe Patriarch::DAOServices::RetrieverService do

  before(:each) do
    @instance = Patriarch::DAOServices::RetrieverService.instance
    @context = {:relation_type => :like, :actor_type => :monster, :target_type => :fallen_angel, 
    :actor_id => 1, :target_id => 1}
    @transaction_item = double()
    @transaction_item.stub(:actor_id).and_return(@context[:actor_id])
    @transaction_item.stub(:actor_type).and_return(@context[:actor_type])
    @transaction_item.stub(:target_type).and_return(@context[:target_type])
    @transaction_item.stub(:target_id).and_return(@context[:target_id])          
  end

  it "returns DAO for actor" do
    @transaction_item.stub(:tripartite?).and_return(false)          
    Patriarch::DAOServices::RedisMapperService.instance.should_receive(:call).twice.with(any_args()).and_return({:type => "sorted_set", :key =>"abc"})
    dao_tab = @instance.call(@transaction_item)

    dao_tab[:actor].key.should == "monster:1:abc"
    dao_tab[:actor].is_a?(Redis::SortedSet).should be_true       
  end

  it "returns DAO for target" do
    @transaction_item.stub(:tripartite?).and_return(false)       
    Patriarch::DAOServices::RedisMapperService.instance.should_receive(:call).twice.with(any_args()).and_return({:type => "sorted_set", :key =>"abc"})
    dao_tab = @instance.call(@transaction_item)

    dao_tab[:target].key.should == "fallen_angel:1:abc"
    dao_tab[:target].is_a?(Redis::SortedSet).should be_true    
  end    

  it "returns DAO for medium" do
    @context.merge! :medium_type => :love_letter, :medium_id => 1
    @transaction_item.stub(:medium_type).and_return(@context[:medium_type])
    @transaction_item.stub(:medium_id).and_return(@context[:medium_id])   
    @transaction_item.stub(:tripartite?).and_return(true)   
    Patriarch::DAOServices::RedisMapperService.instance.should_receive(:call).exactly(3).times.with(any_args()).and_return({:type => "sorted_set", :key =>"abc"})
    dao_tab = @instance.call(@transaction_item)

    dao_tab[:medium].key.should == "love_letter:1:abc"
    dao_tab[:medium].is_a?(Redis::SortedSet).should be_true
  end      

  it "shall reject types that are not suited" do
    Patriarch::DAOServices::RedisMapperService.instance.should_receive(:call).with(any_args()).and_return({:type => "sorted", :key =>"abc"})
    expect{
      @instance.call(@transaction_item)
    }.to raise_error(InvalidRedisTypeException)
  end  
end


    
    
