require 'spec_helper'

describe Patriarch::ToolServices::RedisCleanerService.instance do

  before(:all) do
    Monster.add_behaviour       :lure, :by => [:fallenAngel]
    FallenAngel.add_behaviour   :lure, :on => [:monster]
    Monster.add_behaviour       :like, :on => [:fallenAngel] 
    FallenAngel.add_behaviour   :like, :by => [:monster]

    # Tripartite behaviour ...
    FallenAngel.add_behaviour   :praise, :by => [:monster],     :via => [:loveLetter]
    Monster.add_behaviour       :praise, :on => [:fallenAngel], :via => [:loveLetter]
    LoveLetter.add_behaviour    :praise, :medium_between  =>  [:monster,:fallenAngel]
  end

  let(:monster)      { Monster.create }
  let(:beast)        { Monster.create }
  let(:love_letter)  { LoveLetter.create }
  let(:fallen_angel) { FallenAngel.create }
  # Wrong usage when summoning several times fallen_angel within tests; bad knowledge of 
  # let is in cause i think
  # let(:fallen_angel) { FallenAngel.new }
  let(:instance)     { Patriarch::ToolServices::RedisCleanerService.instance }

  context "#clean_all" do
    it "removes presence into other sets for each behaviour" do
      f = FallenAngel.create
      m = Monster.create
      m.like f
      f.lure m      
      instance.clean_all(f).execute
      f.monsters_i_lure.should be_empty
      f.monsters_liking_me.should be_empty
      m.fallen_angels_i_like.include?(f).should be_false
      m.fallen_angels_luring_me.include?(f).should be_false
    end
  end 

  context "#clean_behaviour" do
    it "removes given behaviour binds on self" do
      f = FallenAngel.create
      f.lure Monster.create
      instance.clean_behaviour(f,:lure).execute
      f.monsters_i_lure.should be_empty
    end

    it "removes given behaviour binds on other entities" do
      f = FallenAngel.create
      m = Monster.create
      b = Monster.create
      f.lure m
      f.lure b    
      instance.clean_behaviour(f,:lure).execute
      b.fallen_angels_luring_me.include?(f).should be_false
      m.fallen_angels_luring_me.include?(f).should be_false      
    end

    it "removes given tripartite behaviour binds on other entities" do
      m = Monster.create
      m.praise love_letter,fallen_angel
      instance.clean_behaviour(m,:praise).execute
      fallen_angel.monsters_praising_me_via_love_letters.include?(m).should be_false
    end    

    it "removes given tripartite behaviour binds on self" do
      m = Monster.create
      m.praise love_letter,fallen_angel
      instance.clean_behaviour(m,:praise).execute
      m.fallen_angels_i_praise_via_love_letters.include?(monster).should be_false
    end        

    it "runs within a unique transaction" do
      f = FallenAngel.create
      10.times { |t| f.lure Monster.create }
      before_count = Patriarch::TransactionServices::TransactionManagerService.instance.count
        instance.clean_behaviour(f,:lure).execute
      after_count = Patriarch::TransactionServices::TransactionManagerService.instance.count
      (after_count - before_count).should eq(1)
    end
  end

end # desc Patriarch::Behaviours::RedisCleanerService.instance
