require 'redis'
require 'redis/objects'

# Beware of poor autoload in redis_objects ...
# TODO ask nateware 
require 'redis/objects/hashes'
require 'redis/objects/sorted_sets'

# Test if we do not need to load gems loaded through dev dependencies into gemspecs
require 'patriarch'
require 'active_record'
require 'action_view'
require 'rails/generators'
require 'rspec/autorun'

RSpec.configure do |config|
  config.alias_it_should_behave_like_to :it_has_behavior, 'has behavior:'
end

# Setting up redis-database
$redis=Redis.new(:host => 'localhost', :port => 6379, :db => 8)
$redis.flushdb

dummy_location = File.expand_path("../../spec/dummy", __FILE__)

puts "Cleaning previous dummy"
# refactor with elegant code here
`yes|rm -r #{dummy_location + "/*"}` 

# Destroy previously initialized files in dummy, clean up !

puts "Gem set up ..."
capture(:stdout) { Patriarch::Generators::InstallGenerator.start(%w(fake_arg),:destination_root => dummy_location) }

puts "Storing files created during behaviour generation in a dummy repository"
# Capture prevents stdout outputs from code between curly braces to be displayed
# This avoid polluting user interface with file generators output
capture(:stdout) { Patriarch::Generators::BehaviourGenerator.start(%w(like),{:destination_root => dummy_location}) }
capture(:stdout) { Patriarch::Generators::BehaviourGenerator.start(%w(lure),{:destination_root => dummy_location}) }
capture(:stdout) { Patriarch::Generators::BehaviourGenerator.start(%w(praise tripartite),{:destination_root => dummy_location}) }

# Enhance load_path with dummy located files ...
$LOAD_PATH << File.expand_path("../../spec/dummy/lib", __FILE__)

# Load files living in the dummy using an initilizer to mimic rails behaviour
require File.expand_path("../../spec/dummy/config/initializers/patriarch", __FILE__)

# We define 3 similar classes to play tests
class Monster     < ActiveRecord::Base; include Patriarch::Behaviours; end
class FallenAngel < ActiveRecord::Base; include Patriarch::Behaviours; end
class LoveLetter  < ActiveRecord::Base; include Patriarch::Behaviours; end

# Setting standalone database to run tests
ActiveRecord::Base.establish_connection(
  :adapter  => "sqlite3",
  :database => ":memory:" #":memory:"
)

# Load minimal schema for database

capture(:stdout) do
  ActiveRecord::Schema.define do
    create_table "fallen_angels", :force => true
    create_table "love_letters", :force => true 
    create_table "monsters", :force => true 
  end
end
